package edu.berkeley.nlp.PCFGLA;

import java.util.regex.Pattern;
import java.util.regex.Matcher;

/**
 * @author igoto, jkawai
 *
 */
public class ConstrainInfo{
	private int[][] subtrees = null;
	private int[][] wallSpans = null;
	private static int modeConstraint = 0;
	private static int modeWall = 0;
	private static double weightIn = 1.0;
	private static double weightOut = 1.0;
	private static double wallPenalty = 1.0;
	private static double logWallPenalty = 0.0;
	private static String regex = "_SW";
	private static Pattern pattern = Pattern.compile(regex);
	//private static double logWeight = 0.0;

	private static int modeSpanConstraint = 0;
	private int spanMaxLen = 0;
	private double[][] spanConstTable = null;

	// check whether the end is "_SW" or not
	//  0:except for _SW, 1:_SW
	private static int check_swap( String POS ) {

		Matcher m = pattern.matcher(POS);

		if (m.find()){
			//System.out.println("POS = " + POS + ", _SW");
			return 1;

		} else {
			//System.out.println("POS = " + POS + ", _ST");
			return 0;
		}
	}

	private int checkBreakSubtree( int start, int end ) {

		int flag_break_subtree = 0;
		int tail = end - 1;

		for ( int i = 0; i < subtrees.length; ++ i ) {
			
			if ( ( subtrees[i][0] < start && start <= subtrees[i][1] && subtrees[i][1] < tail ) ||
					 ( start < subtrees[i][0] && subtrees[i][0] <= tail && tail < subtrees[i][1] )
					 ) {
				flag_break_subtree = 1;
				break;
			}
		}

		return flag_break_subtree;
	}


	public static void setModeConstraint( int m ) {
		modeConstraint = m;
	}
	public static int getModeConstraint() {
		return modeConstraint;
	}

	public static void setModeWall( int m ) {
		modeWall = m;
	}

	public static void setWeightIn( double w ) {
		weightIn = w;
		//logWeight = Math.log(w);
	}

	public static void setWeightOut( double w ) {
		weightOut = w;
	}

	public static void setWallPenalty( double w ) {
		wallPenalty = w;
		logWallPenalty = Math.log(w);
	}

	public void setSubtrees( int[][] st ) {
		subtrees = st;
	}

	public void setWallSpans( int[][] st ) {
		wallSpans = st;
	}

	public boolean isActive() {
		return (subtrees != null || wallSpans != null);
	}

	public int checkRuleSkip( int start, int end ) {

		if ( modeConstraint == 1 && subtrees != null && weightOut < 0.0 && checkBreakSubtree( start, end ) == 1 ) {
			return 1;

		} else {
			return 0;
		}
	}


	public double[][][] calcWeightedScore( int start, int end, boolean logarithmMode, double[][][] scores, double[][][] scores_high, double[][][] scores_low, String parentPOS, double[][][] scores_wallPenalty, double[][][] scores_high_wallPenalty, double[][][] scores_low_wallPenalty ) {

		//System.out.println("1 POS = " + parentPOS);
		//System.out.println("modeWall = " + modeWall);
		//System.out.println("modeConstraint = " + modeConstraint);
		//if ( subtrees == null ) {	System.out.println("subtree = null"); }
		//if ( wallSpans == null ) { System.out.println("wallSpans == null"); }
		//System.out.println("check_swap = " + check_swap( parentPOS ));

		if ( ( modeConstraint == 0 && modeWall == 0 ) ||
				 ( subtrees == null   && wallSpans == null ) ||
				 ( modeWall == 0      && subtrees == null ) ||
				 ( modeConstraint == 0 && wallSpans == null ) ||
				 ( modeWall == 0 && modeConstraint == 1 && weightIn == 1.0 && ( weightOut < 0.0 || weightOut == 1.0 ) )
				 ) {
			return scores;
		}

		//System.out.println("2 POS = " + parentPOS);

		int tail = end - 1;

		double[][][] scores_select = null;
		double[][][] scores_select_wall = scores_wallPenalty;

		int wall_active = 0;

		if ( modeWall > 0 && wallSpans != null ) {

			int flag_break_wallSpan = 0;
			int flag_break_subtee_wallSpan = 0;

			int flag_in_wallSpan = 0;

			for ( int i = 0; i < wallSpans.length; ++ i ) {
				
				if ( wallSpans[i][0] <= start && tail <= wallSpans[i][1] ) {
					flag_in_wallSpan = 1;
					break;
				}
			}

			if ( flag_in_wallSpan == 0 ) {

				int index_last = 0;
				int[] span_v = new int[wallSpans.length];

				for ( int i = 0; i < wallSpans.length; ++ i ) {
					
					if ( ( wallSpans[i][0] <= start && start <= wallSpans[i][1] && wallSpans[i][1] < tail ) ||
							 ( start < wallSpans[i][0] && wallSpans[i][0] <= tail && tail <= wallSpans[i][1] ) ||
							 ( start < wallSpans[i][0] && wallSpans[i][1] < tail )
							 ) {
						span_v[ index_last ] = i;
						++ index_last;
					}
				}

				if ( index_last > 1 ) {
					flag_break_wallSpan = 1;

					if ( wallSpans[ span_v[0] ][0] < start ||
							 tail < wallSpans[ span_v[ index_last - 1 ] ][1] ) {
						flag_break_subtee_wallSpan = 1;
					}
				}
			}


			//System.out.println("start (" + start + "), tail (" + tail + "), flag (" + flag +")");

			if ( ( flag_break_wallSpan == 1 && check_swap( parentPOS ) == 1 ) ||
					 ( modeWall == 2 && flag_break_subtee_wallSpan == 1 )
					 ) {

				wall_active = 1;
			}

		}



		if ( modeConstraint == 1 && subtrees != null ) {

			int flag_break_subtree = checkBreakSubtree( start, end );

			//System.out.println("start (" + start + "), tail (" + tail + "), flag (" + flag +")");

			if ( flag_break_subtree == 0 ) {
				scores_select = scores_high;
				scores_select_wall = scores_high_wallPenalty;
			} else {
				scores_select = scores_low;
				scores_select_wall = scores_low_wallPenalty;
			}

			double weightX = 1.0;
			if ( flag_break_subtree == 0 ) {
				if ( weightIn > 0.0 ) {
					weightX = weightIn;
				}
			} else {
				if ( weightOut > 0.0 ) {
					weightX = weightOut;
				}
			}
			if(weightX == 1.0) {
				scores_select = scores;
			}

			if ( scores_select == null ) {

				// copy score & weighting
				scores_select = new double[scores.length][][];
				for ( int i = 0; i < scores.length; ++ i ) {
			
					if ( scores[i] == null ) continue;
			
					scores_select[i] = new double[scores[i].length][];
			
					for ( int j = 0; j < scores[i].length; ++ j ) {
				
						if ( scores[i][j] == null ) continue;
				
						scores_select[i][j] = new double[scores[i][j].length];
				
						for ( int k = 0; k < scores[i][j].length; ++ k ) {

							double sc = scores[i][j][k];
							if ( logarithmMode == true ) {
								sc *= weightX;
							} else {
								sc = Math.pow( sc, weightX );
							}
							scores_select[i][j][k] = sc;

						}
					}
				}
			}

		}


		if ( scores_select == null ) {
			scores_select = scores;
		}


		if ( wall_active == 1 ) {

			if ( scores_select_wall == null ) {
				// copy score
				scores_select_wall = new double[scores_select.length][][];
				for ( int i = 0; i < scores_select.length; ++ i ) {
			
					if ( scores_select[i] == null ) continue;
			
					scores_select_wall[i] = new double[scores_select[i].length][];
			
					for ( int j = 0; j < scores_select[i].length; ++ j ) {
				
						if ( scores_select[i][j] == null ) continue;
				
						scores_select_wall[i][j] = new double[scores_select[i][j].length];
				
						for ( int k = 0; k < scores_select[i][j].length; ++ k ) {


							if ( logarithmMode == true ) {
								scores_select_wall[i][j][k] = scores_select[i][j][k] + logWallPenalty;

							} else {
								scores_select_wall[i][j][k] = scores_select[i][j][k] * wallPenalty;
							}

						}
					}
				}
			}

			scores_select = scores_select_wall;
		}


		return scores_select;
	}

	public static void setModeSpanConstraint( int m ) {
		modeSpanConstraint = m;
	}

	public static int getModeSpanConstraint() {
		return modeSpanConstraint;
	}

	public void setSpanConstraint( double[][] sct, int maxLen ) {
		spanConstTable = sct;
		spanMaxLen = maxLen;
	}

	public double getSpanConstraint(int first, int last) {
		// as logprob
		return (first > 0 && first < spanMaxLen
				&& last > 0  && last < spanMaxLen)
			? spanConstTable[first][last] : 0.0;

	}
}
